# -*- coding: utf-8 -*-
from __future__ import unicode_literals

title = 'КОС v1'

import sys
if sys.version_info[0] < 3:
	import Tkinter as tk, tkMessageBox
	def decodestr(txt): return txt.decode('cp1251')
else:
	import tkinter as tk, tkinter.messagebox as tkMessageBox
	def decodestr(txt): return txt

try:
	from win32com.client import Dispatch, gencache
	try:
		KompasAPI5 = gencache.EnsureModule('{0422828C-F174-495E-AC5D-D31014DBBE87}', 0, 1, 0)
		KompasAPI7 = gencache.EnsureModule('{69AC2981-37C0-4379-84FD-5DD2F3C0A520}', 0, 1, 0)
	except:
		import KompasAPI5, KompasAPI7
except:
	from traceback import format_exc
	tk.Tk().withdraw()
	tkMessageBox.showerror(title, decodestr(format_exc()))
	sys.exit()

KompasObject = Dispatch('Kompas.Application.5', None, KompasAPI5.KompasObject.CLSID)
iApplication = KompasObject.ksGetApplication7()	#	Dispatch('Kompas.Application.7')
if not KompasObject.Visible: KompasObject.Visible = True
iPropertyMng = KompasAPI7.IPropertyMng(iApplication)							# Интерфейс Менеджера свойств
k14 = True if KompasObject.ksGetSystemVersion(0, 0, 0, 0)[1] >= 14 else False	# Если версия Компас 14 и выше

def geom_wo_pos(BaseObj):														# Получает у объекта спецификации геометрию без позиций
	Geom = BaseObj.Geometry														# Кортеж геометрии объекта спецификации: IPart7, IBody7, IPositionLeader
	if Geom:
		geombody = []
		for i in Geom:															# Перебрать элементы массива геометрии
			GeomiType = KompasAPI7.IKompasAPIObject(i).Type
			if GeomiType == 11000:												# Если геометрия - компонент
				return i
			elif GeomiType == 11009:											# Если геометрия - тело
				geombody.append(i)
		lengeombody = len(geombody)
		if lengeombody == 1:
			return geombody[0]
		elif lengeombody > 1:
			return geombody

def copyos(cut):																# Копирует объекты спецификации
	global table, colcount, addcount, co, tablerow
	iDocument = iApplication.ActiveDocument										# Получим интерфейс активного документа API7
	if iDocument:																# Если есть активный документ
		DocType = iDocument.DocumentType										# Тип документа
		if DocType in (1, 2, 3, 4, 5, 7):										# Если активный документ имеет подходящий тип
			Description = iDocument.SpecificationDescriptions.Active			# Текущее описание спецификации
			if Description:														# Если имеется описание спецификации
				iDoc = KompasObject.TransferInterface(iDocument, 1, 0)			# Получим интерфейс активного документа API5
				if DocType == 3 or iDoc.ksIsSlaveSpcOpened() == 2:				# Если спецификация или открыто и активно окно спецификации в Slave режиме
					Specification = iDoc.GetSpecification()						# Получить указатель на интерфейс для работы с объектами спецификации
					CurrentSpcObject = Specification.ksGetCurrentSpcObject()	# Получить указатель текущего (выделенного или редактируемого) объекта спецификации
					UniqueNumber = Specification.ksGetSpcObjectNumber(CurrentSpcObject)	# Получить уникальный номер текущего объекта спецификации
					if UniqueNumber:											# ksSpecificationBaseObject или ksSpecificationComment
						ShowFull = Description.SpecificationStyle.SpecificationTuning.ShowPerformanceFull if DocType in (4, 5, 7) else Description.SpecificationTuning.ShowPerformanceFull
						i = Description.CurrentObject							# Текущий объект спецификации (выделенный или редактируемый в таблице спецификации)
						SpcObjectType = i.ObjectType							# Тип объекта
						SpcObjectSect = i.Section								# Номер раздела
						SpcObjectSub = i.Subsection								# Номер подраздела
						SpcObjectInc = i.IncrementPosition						# Позиция объекта возрастает
						SpcBaseObject = KompasAPI7.ISpecificationBaseObject(i)	# Интерфейс базового объекта спецификации
						SpcObjectIsp = SpcBaseObject.Performance				# Объект является исполнением
						SpcObjectPos = SpcBaseObject.DrawPosition				# Показывать позицию
						if k14:
							iSystemSettings = iApplication.SystemSettings
							iSystemSettings.EnablesAddSystemDelimersInMarking = True
						if SpcObjectIsp and not ShowFull:
							Description.NeedRebuild = False
							SpcBaseObject.Performance = False					# Чтобы обозначение исполнения прочитать полностью
							SpcBaseObject.Update()
						tablerow = [SpcObjectType, SpcObjectSect, SpcObjectSub, SpcObjectInc, SpcObjectIsp, SpcObjectPos]	# Тип, Раздел, Подраздел, Позиция возрастает, Исполнение, Показывать позицию
						columns, addcols = i.Columns, i.AdditionalColumns
						colcount, addcount = columns.Count, addcols.Count
						prim = ''												# Колонка Примечание может оказаться между первой и последующими колонками Количество
						for j in range(colcount):
							col = columns.Item(j)
							coltype = col.ColumnType
							if coltype == 3 and SpcObjectType == 1 and DocType in (4, 5, 7):	# Если Тип колонки Позиция (ksSColumnPosition) и Тип объекта Базовый и документ-модель
								geomet = geom_wo_pos(SpcBaseObject)				# Получить геометрию без позиций
								if geomet:
									if isinstance(geomet, list):
										tablerow.append(KompasAPI7.IPropertyKeeper(geomet[0]).GetPropertyValue(iPropertyMng.GetProperty(iDocument, 15.0), 0, True)[1])
									else:
										tablerow.append(KompasAPI7.IPropertyKeeper(geomet	).GetPropertyValue(iPropertyMng.GetProperty(iDocument, 15.0), 0, True)[1])
								else:
									tablerow.append(col.Text.Str)
							elif coltype == 6 and SpcObjectType == 1 and SpcObjectSect != 5:	# Если Тип колонки Количество (ksSColumnCount) и Тип объекта Базовый и раздел не Документация
								sc = SpcBaseObject.SummaryCount(col.ColumnTypeNumber, col.BlockNumber)
								tablerow.append(sc if sc else '')
							elif coltype == 7:									# Если Тип колонки Примечание
								prim = col.Text.Str								# Сохранить содержимое ячейки
							else:
								tablerow.append(col.Text.Str)
						tablerow.append(prim)									# Добавить сохраненное Примечание в конец видимых столбцов
						for j in range(addcount):
							tablerow.append(addcols.Item(j).Text.Str)
						if cut:													# Если вырезать
							i.Delete()
							if SpcObjectIsp and not ShowFull:
								Description.NeedRebuild = True
						elif SpcObjectIsp and not ShowFull:
							SpcBaseObject.Performance = True					# Вернуть исходное свойство
							SpcBaseObject.DrawPosition = SpcObjectPos
							SpcBaseObject.Update()
							Description.NeedRebuild = True
						if k14:
							iSystemSettings.EnablesAddSystemDelimersInMarking = False
						pastbut['state'], pastbas['state'] = 'normal', 'normal'
						messages['text'] = 'Объект вырезан' if cut else 'Объект скопирован'
						co = True
						return
				SpcObjects = Description.Objects								# Порядок и состав списка соответствует визуальному отображению данного описания спецификации
				if SpcObjects:													# Если есть объекты спецификации
					co = False
					cutbut['state'], copybut['state'], pastbut['state'], pastbas['state'] = 'disabled', 'disabled', 'disabled', 'disabled'
					iSpecificationStyle = Description.SpecificationStyle
					SecOn = iSpecificationStyle.SectionOn						# Деление на разделы
					ShowFull = iSpecificationStyle.SpecificationTuning.ShowPerformanceFull if DocType in (4, 5, 7) else Description.SpecificationTuning.ShowPerformanceFull
					table, needcount = [], True
					iProperty = iPropertyMng.GetProperty(iDocument, 15.0)		# Получить свойство Позиция 15.0
					k, lenSpcObj = 0, len(SpcObjects)
					action = 'Вырезание ОС: ' if cut else 'Чтение ОС: '
					if k14:
						iSystemSettings = iApplication.SystemSettings
						iSystemSettings.EnablesAddSystemDelimersInMarking = True
					Description.NeedRebuild = False
					for i in SpcObjects:										# Перебрать видимые объекты
						k += 100
						messages['text'] = action + str(k//lenSpcObj) + '%'
						messages.update()
						SpcObjectType = i.ObjectType							# Тип объекта
						if SpcObjectType in (1, 2):								# ksSpecificationBaseObject или ksSpecificationComment
							SpcObjectSect = i.Section							# Номер раздела
							if not SecOn or SpcObjectSect > 5:					# Если нет деления на разделы или объект спецификации после раздела Документация
								SpcObjectSub = i.Subsection						# Номер подраздела
								SpcObjectInc = i.IncrementPosition				# Позиция объекта возрастает
								SpcBaseObject = KompasAPI7.ISpecificationBaseObject(i)	# Интерфейс базового объекта спецификации
								SpcObjectIsp = SpcBaseObject.Performance		# Объект является исполнением
								SpcObjectPos = SpcBaseObject.DrawPosition		# Показывать позицию
								if SpcObjectIsp and not ShowFull:
									SpcBaseObject.Performance = False			# Чтобы обозначение исполнения прочитать полностью
									SpcBaseObject.Update()
								tablerow = [SpcObjectType, SpcObjectSect, SpcObjectSub, SpcObjectInc, SpcObjectIsp, SpcObjectPos]	# Тип, Раздел, Подраздел, Позиция возрастает, Исполнение, Показывать позицию
								columns, addcols = i.Columns, i.AdditionalColumns
								if needcount:
									colcount, addcount, needcount = columns.Count, addcols.Count, False
								prim = ''										# Колонка Примечание может оказаться между первой и последующими колонками Количество
								for j in range(colcount):
									col = columns.Item(j)
									coltype = col.ColumnType
									if coltype == 3 and SpcObjectType == 1 and DocType in (4, 5, 7):	# Если Тип колонки Позиция (ksSColumnPosition) и Тип объекта Базовый и документ-модель
										geomet = geom_wo_pos(SpcBaseObject)		# Получить геометрию без позиций
										if geomet:
											if isinstance(geomet, list):
												tablerow.append(KompasAPI7.IPropertyKeeper(geomet[0]).GetPropertyValue(iProperty, 0, True)[1])
											else:
												tablerow.append(KompasAPI7.IPropertyKeeper(geomet	).GetPropertyValue(iProperty, 0, True)[1])
										else:
											tablerow.append(col.Text.Str)
									elif coltype == 6 and SpcObjectType == 1:	# Если Тип колонки Количество (ksSColumnCount) и Тип объекта Базовый
										sc = SpcBaseObject.SummaryCount(col.ColumnTypeNumber, col.BlockNumber)
										tablerow.append(sc if sc else '')
									elif coltype == 7:							# Если Тип колонки Примечание
										prim = col.Text.Str						# Сохранить содержимое ячейки
									else:
										tablerow.append(col.Text.Str)
								tablerow.append(prim)							# Добавить сохраненное Примечание в конец видимых столбцов
								for j in range(addcount):
									tablerow.append(addcols.Item(j).Text.Str)
								if cut:											# Если вырезать
									i.Delete()
								elif SpcObjectIsp and not ShowFull:
									SpcBaseObject.Performance = True			# Вернуть исходное свойство
									SpcBaseObject.DrawPosition = SpcObjectPos
									SpcBaseObject.Update()
								table.append(tablerow)
					Description.NeedRebuild = True
					if k14:
						iSystemSettings.EnablesAddSystemDelimersInMarking = False
					cutbut['state'], copybut['state'] = 'normal', 'normal'
					if table:
						pastbut['state'] = 'normal'
						messages['text'] = 'Завершено вырезание ОС' if cut else 'Завершено чтение ОС'
					else: messages['text'] = 'БОС и ВОС не найдены'
				else: messages['text'] = 'Отсутствуют объекты'
			else: messages['text'] = 'Нет описания спецификации'
		else: messages['text'] = 'Документ текстовый'
	else: messages['text'] = 'Откройте файл'

def createObject(Description, i):
	global LastBaseObject
	if i[0] == 1:
		NewObject = Description.BaseObjects.Add(i[1], 0.0)
		if NewObject:
			NewObject.Performance = i[4]
			LastBaseObject = NewObject
	else:
		NewObject = Description.CommentObjects.Add(i[1])

	if NewObject:
		columns, addcols = NewObject.Columns, NewObject.AdditionalColumns
		for j in range(colcount):
			column = columns.Item(j)
			if column:
				column.Text.Str = i[j+6] if i[j+6] else ''
		for j in range(addcount):
			addcol = addcols.Item(j)
			if addcol:
				addcol.Text.Str = i[j+6+colcount] if i[j+6+colcount] else ''
		NewObject.Subsection = i[2]
		NewObject.IncrementPosition = i[3]
		NewObject.Update()
		if i[0] == 1 and (i[4] or not i[5]):
			NewObject.DrawPosition = True if i[5] else False
			NewObject.Update()
	return NewObject

def pastos():
	global LastBaseObject
	iDocument = iApplication.ActiveDocument										# Получим интерфейс активного документа API7
	if iDocument:																# Если есть активный документ
		if iDocument.DocumentType in (1, 2, 3, 4, 5, 7):						# Если активный документ имеет подходящий тип
			Description = iDocument.SpecificationDescriptions.Active			# Текущее описание спецификации
			if Description:														# Если имеется описание спецификации
				Description.NeedRebuild = False									# Необходимость перестроения спецификации
				if co:
					Specification = KompasObject.TransferInterface(iDocument, 1, 0).GetSpecification()	# Получить указатель на интерфейс для работы с объектами спецификации
					CurrentSpcObject = Specification.ksGetCurrentSpcObject()	# Получить указатель текущего (выделенного или редактируемого) объекта спецификации
					UniqueNumber = Specification.ksGetSpcObjectNumber(CurrentSpcObject)	# Получить уникальный номер текущего объекта спецификации
					BaseObject = Description.BaseObjects.Item(UniqueNumber)		# Базовый объект спецификации по уникальному номеру
					NewObject = createObject(Description, tablerow)
					if NewObject:
						if BaseObject and BaseObject.Section == tablerow[1] and tablerow[0] == 2:
							BaseObject.CommentObjects.Attach(NewObject)
							BaseObject.Update()
						messages['text'] = 'Объект вставлен'
					else:
						messages['text'] = 'Ошибка создания объекта'
				else:
					copybut['state'], pastbut['state'] = 'disabled', 'disabled'
					LastBaseObject = None
					k, lentable = 0, len(table)
					for i in table:
						k += 100
						messages['text'] = 'Создание ОС: ' + str(k//lentable) + '%'
						messages.update()
						NewObject = createObject(Description, i)
						if LastBaseObject and i[0] == 2:
							LastBaseObject.CommentObjects.Attach(NewObject)
							LastBaseObject.Update()
					copybut['state'], pastbut['state'] = 'normal', 'normal'
					messages['text'] = 'Завершено создание ОС'
				Description.NeedRebuild = True
			else: messages['text'] = 'Нет описания спецификации'
		else: messages['text'] = 'Документ текстовый'
	else: messages['text'] = 'Откройте файл'

def pastas():
	if co:
		a0, a1, a5 = tablerow[0], tablerow[1], tablerow[5]
		tablerow[0], tablerow[1], tablerow[5] = vartype.get(), varsect.get(), varpos.get()
		pastos()
		tablerow[0], tablerow[1], tablerow[5] = a0, a1, a5
	else:
		messages['text'] = 'Только один объект'

def changedtype():
	pos['state'] = 'normal' if vartype.get() == 1 else 'disabled'

def pastmenu():
	global slave, vartype, varsect, varpos, pos

	if 'slave' in globals() and slave.winfo_exists():
		lenx, sep, other = root.geometry().partition('x')
		x, sep, y = other.partition('+')[2].partition('+')
		slave.geometry('+%d+%d' % (int(x) + int(lenx) + 20, int(y) - 100))
		vartype.set(tablerow[0])
		varpos.set(tablerow[5])
		changedtype()
		varsect.set(tablerow[1])
	else:
		slave = tk.Toplevel(root)
		slave.focus()
		slave.title('Вставить как ...')
		slave.wm_attributes('-topmost', 1)
		slave.wm_attributes('-toolwindow', 1)
		slave.resizable(width = False, height = False)
		slave.bind('<Escape>', lambda event: slave.destroy())
		lenx, sep, other = root.geometry().partition('x')
		x, sep, y = other.partition('+')[2].partition('+')
		slave.geometry('+%d+%d' % (int(x) + int(lenx) + 20, int(y) - 100))

		types = ('Базовый', 1), ('Вспомогательный', 2)
		vartype = tk.IntVar()
		vartype.set(tablerow[0])
		for txt, z in types:
			tk.Radiobutton(slave, text = txt, variable = vartype, value = z, command = changedtype).pack(anchor = 'w')

		varpos = tk.BooleanVar()
		varpos.set(tablerow[5])
		pos = tk.Checkbutton(slave, text = 'Показывать позицию', variable = varpos)
		pos.pack(anchor = 'w')
		changedtype()

		tk.Label(slave, text = '———————————').pack()

		sections = ('Документация', 5), ('Комплексы', 10), ('Сборочные единицы', 15), ('Детали', 20), ('Стандартные изделия', 25), ('Прочие изделия', 30), ('Материалы', 35), ('Комплекты', 40)
		varsect = tk.IntVar()
		varsect.set(tablerow[1])
		for txt, z in sections:
			tk.Radiobutton(slave, text = txt, variable = varsect, value = z).pack(anchor = 'w')

		tk.Button(slave, text = 'Вставить', cursor = 'hand2', command = pastas, bg = '#FFC0CB').pack(fill = 'x')

def decompose():
	iDocument2D = KompasObject.ActiveDocument2D()
	if iDocument2D:
		ref = iDocument2D.ksGetReferenceDocumentPart(4)							# 4 - спецификация на листе
		if ref:
			iDocument2D.ksStoreTmpGroup(iDocument2D.ksDecomposeObj(ref, 0, 0, 0))
			messages['text'] = 'Копия СП создана'
		else:
			messages['text'] = 'Нужна СП на листе'
	else:
		messages['text'] = 'Нужен чертеж'

root = tk.Tk()
root.title(title)
root.geometry('+%d+%d' %(root.winfo_screenwidth()/2, root.winfo_screenheight()/2))
root.resizable(width = False, height = False)
root.wm_attributes('-topmost', 1)
root.bind('<Escape>', lambda event: root.quit())

cutbut = tk.Button(root, text = 'Вырезать', cursor = 'hand2', command = lambda: copyos(True), width = 23)
cutbut.pack(fill = 'x')

copybut = tk.Button(root, text = 'Копировать', cursor = 'hand2', command = lambda: copyos(False))
copybut.pack(fill = 'x')

pastbut = tk.Button(root, text = 'Вставить', cursor = 'hand2', command = pastos, state = 'disabled')
pastbut.pack(fill = 'x')

pastbas = tk.Button(root, text = 'Вставить как ...', cursor = 'hand2', command = pastmenu, state = 'disabled')
pastbas.pack(fill = 'x')

decobut = tk.Button(root, text = 'Разрушенная копия', cursor = 'hand2', command = decompose)
decobut.pack(fill = 'x')

messages = tk.Label(root, text = 'Все или один ОС', bg = '#E6E6FA')
messages.pack(fill = 'x')

root.mainloop()